package de.intevation.flys.client.client.ui;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;

import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Window;

import de.intevation.flys.client.client.Config;
import de.intevation.flys.client.client.FLYSConstants;

import de.intevation.flys.client.client.event.DatacageHandler;

import de.intevation.flys.client.client.services.LoadArtifactService;
import de.intevation.flys.client.client.services.LoadArtifactServiceAsync;

import de.intevation.flys.client.shared.model.Artifact;
import de.intevation.flys.client.shared.model.ArtifactDescription;
import de.intevation.flys.client.shared.model.Collection;
import de.intevation.flys.client.shared.model.Data;
import de.intevation.flys.client.shared.model.DataItem;
import de.intevation.flys.client.shared.model.DataList;
import de.intevation.flys.client.shared.model.ToLoad;
import de.intevation.flys.client.shared.model.User;
import de.intevation.flys.client.shared.model.Recommendation;

import java.util.List;


public class DatacageWindow
extends      Window
implements   DatacageHandler
{
    protected FLYSConstants messages =
        GWT.create(FLYSConstants.class);

    protected LoadArtifactServiceAsync loadService =
        GWT.create(LoadArtifactService.class);

    protected CollectionView view;

    public DatacageWindow(
        Artifact       artifact, 
        User           user,
        String         outs,
        CollectionView view
    ) {
        this.view = view;
        setWidth(400);
        setHeight(500);

        DatacageWidget dw = new DatacageWidget(artifact, user, outs);
        dw.addDatacageHandler(this);

        addItem(dw);

        String river =  findRiver(artifact);
        // TODO: i18n
        setTitle("Datenkorb: " + river);
        setShowMinimizeButton(false);
        setIsModal(true);
        setShowModalMask(true);
        setCanDragResize(true);

        centerInPage();
    }


    @Override
    public void toLoad(ToLoad toLoad) {
        destroy();
        List<Recommendation> recs = toLoad.toRecommendations();
        loadArtifacts(recs.toArray(new Recommendation[recs.size()]));
    }


    protected String findRiver(Artifact artifact) {
        ArtifactDescription adescr = artifact.getArtifactDescription();
        DataList [] data = adescr.getOldData();

        if (data != null && data.length > 0) {
            for (int i = 0; i < data.length; i++) {
                DataList dl = data[i];
                if (dl.getState().equals("state.winfo.river")) {
                    for (int j = dl.size()-1; j >= 0; --j) {
                        Data d = dl.get(j);
                        DataItem [] di = d.getItems();
                        if (di != null && di.length == 1) {
                           return d.getItems()[0].getStringValue();
                        }
                    }
                }
            }
        }

        return "";
    }


    protected void loadArtifacts(Recommendation[] recommendations) {
        Config cfg = Config.getInstance();

        final Collection collection     = view.getCollection();
        final Artifact   masterArtifact = view.getArtifact();
        final String     serverUrl      = cfg.getServerUrl();
        final String     locale         = cfg.getLocale();

        for (final Recommendation recommendation: recommendations) {
            // XXX: UGLY! If no reference artifact given use uuid of
            //      current artifact as reference.
            if (recommendation.getMasterArtifact() == null) {
                recommendation.setMasterArtifact(masterArtifact.getUuid());
            }

            final String factory = recommendation.getFactory();

            GWT.log("Load new artifact with factory: " + factory);

            loadService.load(
                collection,
                recommendation,
                factory,
                serverUrl,
                locale,
                new AsyncCallback<Artifact>() {
                    public void onFailure(Throwable caught) {
                        SC.warn(caught.getMessage());
                    }

                    public void onSuccess(Artifact artifact) {
                        GWT.log("Created new artifact: " + artifact.getUuid());
                    }
            });
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
